function [h, missTrace] = run_cache(requests, cache_size)
%RUN_CACHE Summary of this function goes here
% Input:
% - requests -> Vector of time-ordered requests [time;id]
% - size of cache
%
% Output:
% - h -> Per item cache hit rate
% - missTrace -> Miss stream requests (next level input)

%% Init Cache
% Each element of CHP is the tupla -> |number of request, number of hits|

N = numel(unique(requests(:,2)));
CHP = zeros(N, 2);
C = zeros(cache_size, 1);
Nreq = length(requests(:,1));
missTrace = NaN(Nreq, 2);

%% Run cache
idx = 1;
for i=1:Nreq
    id_curr = requests(i,2);
    
    % update statistics
    % cache lookup
    CHP(id_curr,1) = CHP(id_curr,1) + 1;
    k=find(C(:,1)==id_curr);
    if (numel(k)~=0) % Cache hit
        CHP(id_curr,2) = CHP(id_curr,2) + 1;
        C = LruAdd(C,id_curr);
    else % Cache miss
        missTrace(idx, :) = requests(i,:);
        idx = idx + 1;
        C = LruAdd(C,id_curr);
    end
end

%% Outputs
h = CHP(:, 2)./CHP(:, 1); % Compute per-item cache hit
cut_idx = find(isnan(missTrace(:, 1)), 1, 'first');
if ~isempty(cut_idx)
    missTrace = missTrace(1:cut_idx-1, :); % Cut the miss stream trace
end


